# arr-map [![NPM version](https://badge.fury.io/js/arr-map.svg)](http://badge.fury.io/js/arr-map)  [![Build Status](https://travis-ci.org/jonschlinkert/arr-map.svg)](https://travis-ci.org/jonschlinkert/arr-map) 

> Faster, node.js focused alternative to JavaScript's native array map.

JavaScript's native `Array.map()` is slow, and other popular array map libraries are focused on browser compatibility. This implementation is focused on node.js usage keeping it light and fast.

## Install with [npm](npmjs.org)

```bash
npm i arr-map --save
```

## Usage

```js
var map = require('arr-map');

map(['a', 'b', 'c'], function (ele) {
  return ele + ele;
});
//=> ['aa', 'bb', 'cc']

map(['a', 'b', 'c'], function (ele, i) {
  return i + ele;
});
//=> ['0a', '1b', '2c']
```

## Run benchmarks

Install dev dependencies:

```bash
node i -d && npm benchmark
```

[Benchmarks](./benchmark) from March 24, 2015

```bash
#1: long.js
  arr-map.js x 17,326 ops/sec ±0.74% (90 runs sampled)
  array-map.js x 15,219 ops/sec ±0.72% (97 runs sampled)
  native.js x 14,985 ops/sec ±0.66% (93 runs sampled)

#2: med.js
  arr-map.js x 356,497 ops/sec ±0.79% (90 runs sampled)
  array-map.js x 285,448 ops/sec ±0.78% (95 runs sampled)
  native.js x 284,534 ops/sec ±0.82% (97 runs sampled)

#3: short.js
  arr-map.js x 1,576,773 ops/sec ±0.72% (93 runs sampled)
  array-map.js x 1,129,016 ops/sec ±0.67% (97 runs sampled)
  native.js x 1,141,300 ops/sec ±0.71% (97 runs sampled)
```

## Run tests
Install dev dependencies.

```bash
npm i -d && npm test
```

## Contributing
Pull requests and stars are always welcome. For bugs and feature requests, [please create an issue](https://github.com/jonschlinkert/arr-map/issues)

## Related
* [arr-flatten](https://github.com/jonschlinkert/arr-flatten): Recursively flatten an array or arrays. This is the fastest implementation of array flatten.
* [arr-diff](https://github.com/jonschlinkert/arr-diff): Returns an array with only the unique values from the first array, by excluding all values from additional arrays using strict equality for comparisons.
* [arr-filter](https://github.com/jonschlinkert/arr-filter): Faster alternative to javascript's native filter method.
* [arr-reduce](https://github.com/jonschlinkert/arr-reduce): Fast array reduce that also loops over sparse elements.

## Author

**Jon Schlinkert**
 
+ [github/jonschlinkert](https://github.com/jonschlinkert)
+ [twitter/jonschlinkert](http://twitter.com/jonschlinkert) 

## License
Copyright (c) 2015 Jon Schlinkert  
Released under the MIT license

***

_This file was generated by [verb-cli](https://github.com/assemble/verb-cli) on March 24, 2015._